﻿/**
* Sample code for the GetMerchantRegistrationToken Canada Post service.
* 
* The GetMerchantRegistrationToken service returns a unique registration token that is used
* to launch a merchant into the Canada Post sign-up process.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetMerchantRegistrationToken
{
    class GetMerchantRegistrationToken
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_Platforms_DotNet_Samples\SOAP\platforms\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            MerchantRegistrationPortTypeClient server = new MerchantRegistrationPortTypeClient("MerchantRegistrationPort", "https://ct.soa-gw.canadapost.ca/ot/soap/merchant/registration");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getmerchantregistrationtokenrequest request = new getmerchantregistrationtokenrequest();
            request.locale = "FR";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getmerchantregistrationtokenresponse response = server.GetMerchantRegistrationToken(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getmerchantregistrationtokenresponseToken))
                {
                    // Retrieve values from response object
                    getmerchantregistrationtokenresponseToken token = (getmerchantregistrationtokenresponseToken)response.Item;
                    responseAsString += "Token Id: " + token.tokenid + "\r\n";
                }
                else
                {
                    MessagesType msgs = (MessagesType)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
