﻿/**
* Sample code for the GetOpenReturnTemplate Canada Post service.
* 
* The GetOpenReturnTemplate service is used to retrieve the information that 
* allows access to the Retrieve Next Open Return Artifact service associated with
* the template.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetOpenReturnTemplate
{
    class GetOpenReturnTemplate
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Returns_DotNet_Samples\SOAP\returns\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            OpenReturnPortTypeClient server = new OpenReturnPortTypeClient("OpenReturnPort", "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getopenreturntemplaterequest request = new getopenreturntemplaterequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.templateid = "349641323786705649";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getopenreturntemplateresponse response = server.GetOpenReturnTemplate(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(OpenReturnInfoType))
                {
                    OpenReturnInfoType openReturnInfo = (OpenReturnInfoType)response.Item;
                    responseAsString += "Artifacts Remaining: " + openReturnInfo.artifactsremaining + "\r\n";
                    responseAsString += "Template Id: " + openReturnInfo.templateid + "\r\n";
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
