﻿/**
* Sample code for the GetOpenReturnTemplates Canada Post service.
* 
* The GetOpenReturnTemplates service is used to retrieve the list of active open
* return templates (based on the mailed-on-behalf-of customer number). Active
* open return templates are those that still have open return artifacts available
* for retrieval. Once all of the artifacts from a template have been retrieved,
* the open return template is consider depleted, and is deleted. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetOpenReturnTemplates
{
    class GetOpenReturnTemplates
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Returns_DotNet_Samples\SOAP\returns\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            OpenReturnPortTypeClient server = new OpenReturnPortTypeClient("OpenReturnPort", "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getopenreturntemplatesrequest request = new getopenreturntemplatesrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;
            request.fromSpecified = true;
            request.from = new DateTime(2011, 01, 01);
            request.toSpecified = true;
            request.to = new DateTime(2013, 01, 01);

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getopenreturntemplatesresponse response = server.GetOpenReturnTemplates(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(OpenReturnTemplatesInfoType))
                {
                    OpenReturnTemplatesInfoType templates = (OpenReturnTemplatesInfoType)response.Item;
                    if (templates.templateid != null)
                    {
                        foreach (var templateId in templates.templateid)
                        {
                            responseAsString += "Template Id: " + templateId + "\r\n";
                        }
                    }
                    else
                    {
                        responseAsString += "No templates returned.\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
